/*
 * File  : DTPA-UART-3232_ex.c:
 * version : 1.0.0
 * Date : 2022. 5. 25.
 * Contact : expoeb2@diwell.com 
 *
 * description : 
 *   DTPA-UART-3232 example code.  
 *   (using wiringPi Library)
 *
 * Further information:
 *   https://projects.drogon.net/raspberry-pi/wiringpi/
 * This program is distributed but WITHOUT ANY WARRANTY.
 ***********************************************************************
 * Development environment(hardware):
 *   Raspberry Pi 2 Model B v1.1
 ***********************************************************************
 * Revision history.
 * 1. 2022.5.25 : First version is released.
 ***********************************************************************
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <wiringPi.h>
#include <wiringSerial.h>
#include "desc.h"

#define MAX_BUFFER 	2054		// RX varaiale size
#define REQUEST_CYCLE 	500		// 500ms : Request cycle. >500ms
#define TIMEOUT		260		// 260ms : fixed
#define TRANSMISSION_TIME	5	// 5ms : fixed

query_t telegram;			// Master query structure

uint8_t *ptr, com_result;
uint8_t RX_BUF[MAX_BUFFER];
int16_t a16data[(MAX_BUFFER>>1)-4];
uint16_t Request_interval = REQUEST_CYCLE-TIMEOUT-TRANSMISSION_TIME; // 500ms-260ms-5ms = 235ms
int32_t fd;
uint32_t Start_ms;
float Ambient;

int main ()
{	
   	telegram.u8start = 0x11;                         // START : 0x11
   	telegram.u16RegAdd = 0;                       // Start Address : 0
	telegram.u16RegNo = 1025;                    // No. of register : 1025 (all data)
	telegram.u8end = 0x98;		       // End : 0x98
	telegram.au16reg = a16data;                    // pointer to a memory array in the raspberry 

	if ((fd = serialOpen ("/dev/ttyAMA0", 115200)) < 0)
	{
		fprintf (stderr, "Unable to open serial device: %s\n", strerror (errno)) ;
		return 1 ;
	}
	if (wiringPiSetup () == -1)
	{
		fprintf (stdout, "Unable to start wiringPi: %s\n", strerror (errno)) ;
		return 1 ;
	}
	/****************************************************************************************************
	   Notice!! (Sensor initializaion time.)
            It is strongly recommended to wait over 30 seconds for stable data.
	****************************************************************************************************/
        	delay(1000);               			// waiting for Sensor init.(1000ms for test)    			

	while(1)  					
 	{
		com_result = query(telegram);		// communication.

    		if(com_result == COM_OK)
		{
	
			printf("Ta: %3.1f\n", Ambient);	// print ambitent temperature
			printf("pixel Temperature: \n");
			printarray(0, a16data, 1024);		// print pixel temperature
		}
		else if(com_result == COM_ERROR)
		{
			printf("COM Error.\n");
		}
		else if(com_result == NO_REPLY)
		{
			printf("Timeout.\n");
		}
		
		Start_ms = millis();
		while( millis()-Start_ms < Request_interval); 		// Delay : next request

		
 	}
 	//printf ("================Stopped================\n") ;
  	return 0 ;
}

int8_t query(query_t telegram)
{
	uint8_t au8Buffer[6], tx_count,response_status = NO_REPLY;
	uint16_t i,rx_count=0, crc16, *au16regs;

	au16regs = telegram.au16reg;
 	au8Buffer[ 0 ]  = telegram.u8start;
    	au8Buffer[ 1 ]  = highByte(telegram.u16RegAdd );
    	au8Buffer[ 2 ]  = lowByte( telegram.u16RegAdd );
	au8Buffer[ 3 ]  = highByte(telegram.u16RegNo );
        	au8Buffer[ 4 ]  = lowByte( telegram.u16RegNo );
	au8Buffer[ 5 ]  = telegram.u8end;
        

	fflush (stdout) ;			// flush buffer
	ptr = au8Buffer;
	
	for(tx_count=0; tx_count<6; tx_count++)
	{
		serialPutchar (fd, *ptr++) ; 	// transfer data
	}
	// waiting transmission time.
	Start_ms = millis();
	while( millis()-Start_ms < TRANSMISSION_TIME);
	// a receive timeout
	Start_ms = millis();
	while( millis()-Start_ms < TIMEOUT);
	
	while (serialDataAvail (fd))
	{ 
		RX_BUF[rx_count] = serialGetchar (fd);

		switch(rx_count)
		{
		case 0:
			if(RX_BUF[rx_count] == 0x16){rx_count++;}
			break;
		case 1:
			if(RX_BUF[rx_count] == 0x98){rx_count++;}
			else{rx_count=0;response_status = COM_ERROR;}
			break;
		case 2 ... 2051:
			rx_count++;
			break;
		case 2052:
			if(RX_BUF[rx_count] == 0x1A){rx_count++;}
			else{rx_count=0;response_status = COM_ERROR;}
			break;
		case 2053:
			if(RX_BUF[rx_count] == 0x9C)
			{
				response_status = COM_OK;
				
				Ambient = (float)((uint16_t)RX_BUF[2]<<8 | (uint16_t)RX_BUF[3])/10.0;
				for(i=0; i<1025; i++)
				{
					*au16regs = RX_BUF[2*i+4]<<8;
					*au16regs++ |= RX_BUF[2*i+5];
				}

			}
			else
			{
				response_status = COM_ERROR;
			}
			
			rx_count=0;   
			break;
		default: rx_count=0; break;

		} // end switch(rx_count)

	} //end while (serialDataAvail (fd))

	return response_status;
}

uint8_t highByte(uint16_t Data)
{
	return (uint8_t)((Data>>8)&0xFF);
}

uint8_t lowByte(uint16_t Data)
{
	return (uint8_t)(Data&0xFF);
}

void printarray(uint16_t icount, int16_t *p, uint16_t size) {
	if(size == 0) 
	{
		printf("\n");
     		return;
	}
	if(icount == 0)
	{
		printf("%4dp(%3.1f)\n" , icount, *p/10.0);
	}
	else
	{
		if(icount%7 == 0)
		{
			printf("%4dp(%3.1f)\n" , icount, *p/10.0);
		}
	   	else
		{
			printf("%4dp(%3.1f) " , icount, *p/10.0);
		}
	}	
	printarray(++icount, ++p, --size );

}

